﻿// 32feet.NET - Personal Area Networking for .NET
//
// Copyright (c) 2008-2010 In The Hand Ltd, All rights reserved.
// Copyright (c) 2008-2010 Alan J. McFarlane, All rights reserved.
// This source code is licensed under the In The Hand Community License - see License.txt

using List_IBluetoothDeviceInfo = System.Collections.Generic.List<InTheHand.Net.Bluetooth.Factory.IBluetoothDeviceInfo>;
using AsyncResultDD = InTheHand.Net.AsyncResult<
    System.Collections.Generic.List<InTheHand.Net.Bluetooth.Factory.IBluetoothDeviceInfo>,
    InTheHand.Net.Bluetooth.Factory.DiscoDevsParams>;
//
using System;
using System.Diagnostics;
using System.Net.Sockets;
using InTheHand.Net.Sockets;

namespace InTheHand.Net.Bluetooth.Factory
{
    /// <exclude/>
    public abstract class CommonDiscoveryBluetoothClient : IBluetoothClient
    {
        // !!!
#pragma warning disable 1591 // Missing XML comment for publicly visible type or member '
        //
        TimeSpan _inquiryLength = TimeSpan.FromSeconds(20);

        protected CommonDiscoveryBluetoothClient()
        {
        }

        //----------------------------------------
        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            // TODO Abort/Disallow device discovery
        }

        //----------------------------------------
        public IBluetoothDeviceInfo[] DiscoverDevices(int maxDevices,
            bool authenticated, bool remembered, bool unknown, bool discoverableOnly)
        {
            IAsyncResult ar = BeginDiscoverDevices(maxDevices,
                authenticated, remembered, unknown, discoverableOnly,
                null, null);
            return EndDiscoverDevices(ar);
        }

        public IAsyncResult BeginDiscoverDevices(int maxDevices,
            bool authenticated, bool remembered, bool unknown, bool discoverableOnly,
            AsyncCallback callback, object state)
        {
            return BeginDiscoverDevices(maxDevices,
                authenticated, remembered, unknown, discoverableOnly,
                callback, state,
                null, null);
        }

        public IAsyncResult BeginDiscoverDevices(int maxDevices,
            bool authenticated, bool remembered, bool unknown, bool discoverableOnly,
            AsyncCallback callback, object state,
            BluetoothClient.LiveDiscoveryCallback liveDiscoHandler, object liveDiscoState)
        {
            DateTime discoTime = DateTime.UtcNow;
            DiscoDevsParams args = new DiscoDevsParams(maxDevices, authenticated, remembered, unknown, discoverableOnly, discoTime);
            AsyncResultDD arDD = new AsyncResultDD(callback, state, args);
            //
            if (unknown || discoverableOnly) { // No need to do SLOW Inquiry when just want known remembered.
                BeginInquiry(maxDevices, DiscoDevs_InquiryCallback, arDD, liveDiscoHandler, liveDiscoState);
            } else {
                arDD.SetAsCompleted(null, true);
            }
            return arDD;
        }

        protected abstract void BeginInquiry(int maxDevices,
            AsyncCallback callback, object state,
            BluetoothClient.LiveDiscoveryCallback liveDiscoHandler, object liveDiscoState);

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        void DiscoDevs_InquiryCallback(IAsyncResult ar)
        {
            AsyncResultDD arDD = (AsyncResultDD)ar.AsyncState;
            try {
                List_IBluetoothDeviceInfo discoverableDevices = EndInquiry(ar);
                arDD.SetAsCompleted(discoverableDevices, false);
            } catch (Exception ex) {
                arDD.SetAsCompleted(ex, false);
            }
        }

        protected abstract List_IBluetoothDeviceInfo EndInquiry(IAsyncResult ar);

        public IBluetoothDeviceInfo[] EndDiscoverDevices(IAsyncResult asyncResult)
        {
            AsyncResultDD arDD = (AsyncResultDD)asyncResult;
            List_IBluetoothDeviceInfo discoverableDevices = arDD.EndInvoke();
            DiscoDevsParams args = arDD.BeginParameters;
            // DEBUG Iff 'known' devices only: we complete immediately (sync'ly), and with null result.
            if (args.unknown || args.discoverableOnly) {
                // Result from BeginInquiry callback expected
                Debug.Assert(discoverableDevices != null, "a1");
                Debug.Assert(!arDD.CompletedSynchronously, "a2"); // don't really care however
            } else {
                // Null result from BeginDD method expected.
                Debug.Assert(discoverableDevices == null, "b1");
                Debug.Assert(arDD.CompletedSynchronously, "b2");
            }
            //
            List_IBluetoothDeviceInfo knownDevices = GetKnownRemoteDeviceEntries();
            //
            List_IBluetoothDeviceInfo mergedDevices = BluetoothClient.DiscoverDevicesMerge(
                args.authenticated, args.remembered, args.unknown, knownDevices, discoverableDevices,
                args.discoverableOnly, args.discoTime);
            return mergedDevices.ToArray();
        }

        protected abstract List_IBluetoothDeviceInfo GetKnownRemoteDeviceEntries();

        public TimeSpan InquiryLength
        {
            get { return _inquiryLength; }
            set
            {
                if ((value.TotalSeconds > 0) && (value.TotalSeconds <= 60)) {
                    _inquiryLength = value;
                } else {
                    throw new ArgumentOutOfRangeException("value",
                        "QueryLength must be a positive timespan between 0 and 60 seconds.");
                }
            }
        }

        public int InquiryAccessCode
        {
            get { throw new NotSupportedException(); }
            set { throw new NotSupportedException(); }
        }

        //--------------------------------------------------------------
        public abstract void Connect(BluetoothEndPoint remoteEP);
        public abstract IAsyncResult BeginConnect(BluetoothEndPoint remoteEP, AsyncCallback requestCallback, object state);
        public abstract void EndConnect(IAsyncResult asyncResult);
        //
        public abstract System.Net.Sockets.NetworkStream GetStream();
        public abstract LingerOption LingerState { get; set; }
        public abstract bool Connected { get; }
        //
        public abstract int Available { get; }
        public abstract System.Net.Sockets.Socket Client { get; set; }
        public abstract bool Authenticate { get; set; }
        public abstract bool Encrypt { get; set; }
        public abstract BluetoothEndPoint RemoteEndPoint { get; }
        public abstract string GetRemoteMachineName(BluetoothAddress device);
        public abstract Guid LinkKey { get; }
        public abstract LinkPolicy LinkPolicy { get; }
        public abstract string RemoteMachineName { get; }
        public abstract void SetPin(string pin);
        public abstract void SetPin(BluetoothAddress device, string pin);
    }
}
