﻿// 32feet.NET - Personal Area Networking for .NET
//
// Net.Bluetooth.Factory.CommonBluetoothInquiry
// 
// Copyright (c) 2008-2010 Alan J.McFarlane, All rights reserved.
// Copyright (c) 2008-2010 In The Hand Ltd, All rights reserved.
// This source code is licensed under the In The Hand Community License - see License.txt

using System;
using System.Collections.Generic;
using System.Diagnostics;
using InTheHand.Net.Sockets;
using System.Text;

namespace InTheHand.Net.Bluetooth
{
    public abstract class BluetoothWin32RadioEventArgs : EventArgs
    {
#pragma warning disable 1591 // One XmlDocs warning will do!

        readonly WindowsBluetoothDeviceInfo _bdiWin;
        readonly BluetoothDeviceInfo _bdi;

        //--------
        internal /*and protected*/ BluetoothWin32RadioEventArgs(BLUETOOTH_DEVICE_INFO bdi0)
        {
            _bdiWin = new WindowsBluetoothDeviceInfo(bdi0);
            _bdi = new BluetoothDeviceInfo(_bdiWin);
        }

        //----
        public BluetoothDeviceInfo Device
        {
            get { return _bdi; }
        }

        internal WindowsBluetoothDeviceInfo DeviceWindows
        {
            get { return _bdiWin; }
        }

    }


    public class BluetoothWin32RadioOutOfRangeEventArgs : BluetoothWin32RadioEventArgs
    {
        private BluetoothWin32RadioOutOfRangeEventArgs(BLUETOOTH_DEVICE_INFO bdi0)
            : base(bdi0)
        {
        }

        //----
        internal static BluetoothWin32RadioOutOfRangeEventArgs Create(long addrLong)
        {
            BLUETOOTH_DEVICE_INFO bdi0 = new BLUETOOTH_DEVICE_INFO(addrLong);
            return new BluetoothWin32RadioOutOfRangeEventArgs(bdi0);
        }

        //----
        public override string ToString()
        {
            string s = string.Format(System.Globalization.CultureInfo.InvariantCulture,
                "Device: {0} '{1}'",
                Device.DeviceAddress, Device.DeviceName);
            return s;
        }

    }


    public class BluetoothWin32RadioInRangeEventArgs : BluetoothWin32RadioEventArgs
    {
        readonly BluetoothDeviceInfoProperties _currentFlags, _previousFlags;

        //----
        private BluetoothWin32RadioInRangeEventArgs(BLUETOOTH_DEVICE_INFO bdi0, BluetoothDeviceInfoProperties currentFlags, BluetoothDeviceInfoProperties previousFlags)
            :base(bdi0)
        {
            _currentFlags = currentFlags;
            _previousFlags = previousFlags;
        }

        //----
        internal static BluetoothWin32RadioInRangeEventArgs Create(
            BluetoothDeviceInfoProperties previousFlags,
            BluetoothDeviceInfoProperties currentFlags,
            BLUETOOTH_DEVICE_INFO deviceInfo)
        {
            var e = new BluetoothWin32RadioInRangeEventArgs(deviceInfo, currentFlags, previousFlags);
            return e;
        }

        //----
        public BluetoothDeviceInfoProperties CurrentState
        {
            get { return _currentFlags; }
        }

        public BluetoothDeviceInfoProperties PreviousState
        {
            get { return _previousFlags; }
        }

        BluetoothDeviceInfoProperties DifferentStates
        {
            get { return _currentFlags ^ _previousFlags; }
        }

        public BluetoothDeviceInfoProperties GainedStates
        {
            get { return DifferentStates & _currentFlags; }
        }

        public BluetoothDeviceInfoProperties LostStates
        {
            get { return DifferentStates & _previousFlags; }
        }

        //----
        public override string ToString()
        {
            string s = string.Format(System.Globalization.CultureInfo.InvariantCulture,
                "Device: {0} '{1}', cur flags: '{2}', old: '{3}'",
                Device.DeviceAddress, Device.DeviceName,
                _currentFlags, _previousFlags);
            return s;
        }

    }
}
