﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using Microsoft.DirectX.DirectInput;
using Microsoft.DirectX;

namespace RobotCtrl
{
    public class JoystickInputController
    {
        private static Microsoft.DirectX.DirectInput.JoystickState joystickState = new Microsoft.DirectX.DirectInput.JoystickState();

        private Device applicationDevice = null;
        private static System.Threading.Timer timer;

        public int XAxis1 { get { return joystickState.X; } }
        public int YAxis1 { get { return joystickState.Y; } }
        public int ZAxis1 { get { return joystickState.Z; } }

        public int XAxis2 { get { return joystickState.Rx; } }
        public int YAxis2 { get { return joystickState.Ry; } }
        public int ZAxis2 { get { return joystickState.Rz; } }

        public byte[] Buttons { get { return joystickState.GetButtons(); } }

        private int pollingInterval = 100;
        public int PollingInterval { get { return pollingInterval; }
                                     set { pollingInterval = value; } }

        public void SetVibration()
        {
            
        }

        public event UpdateEventHandler UpdateEvent;
        public delegate void UpdateEventHandler(JoystickInputController controller);

        public JoystickInputController()
        {
        }

        public void ConnectInputDevice(string deviceName)
        {
            // Enumerate joysticks in the system.
            foreach (DeviceInstance instance in Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly))
            {
                // Find the selected device
                if (instance.InstanceName == deviceName)
                {
                    applicationDevice = new Device(instance.InstanceGuid);
                    break;
                }
            }

            if (null == applicationDevice)
            {
                //throw new Exception("Unable to create a joystick device.");
                return;
            }

            // Set the data format to the c_dfDIJoystick pre-defined format.
            applicationDevice.SetDataFormat(DeviceDataFormat.Joystick);
            // Set the cooperative level for the device.
            //applicationDevice.SetCooperativeLevel(this, CooperativeLevelFlags.Exclusive | CooperativeLevelFlags.Foreground);
            // Enumerate all the objects on the device.
            foreach (DeviceObjectInstance d in applicationDevice.Objects)
            {
                // For axes that are returned, set the DIPROP_RANGE property for the
                // enumerated axis in order to scale min/max values.

                if ((0 != (d.ObjectId & (int)DeviceObjectTypeFlags.Axis)))
                {
                    // Set the range for the axis.
                    applicationDevice.Properties.SetRange(ParameterHow.ById, d.ObjectId, new InputRange(-100, +100));
                }
            }

            //Use a threading timer to poll the input controller device periodically
            System.Threading.TimerCallback cb = new System.Threading.TimerCallback(PollDevice);
            timer = new System.Threading.Timer(cb, null, 0, pollingInterval);
        }

        public static Dictionary<Guid, string> EnumDevices()
        {
            Dictionary<Guid, string> values = new Dictionary<Guid, string>();

            // Enumerate joysticks in the system.
            foreach (DeviceInstance instance in Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly))
            {
                values.Add(instance.InstanceGuid, instance.InstanceName);
            }
            return values;
        }


        public void PollDevice(object paramOfTimer)
        {
                // Make sure there is a valid device.
                if (null == applicationDevice)
                    return;

                try
                {
                    // Poll the device for info.
                    applicationDevice.Poll();
                }
                catch (InputException inputex)
                {
                    if ((inputex is NotAcquiredException) || (inputex is InputLostException))
                    {
                        // Check to see if either the app
                        // needs to acquire the device, or
                        // if the app lost the device to another
                        // process.
                        try
                        {
                            // Acquire the device.
                            applicationDevice.Acquire();
                        }
                        catch (InputException)
                        {
                            // Failed to acquire the device.
                            // This could be because the app
                            // doesn't have focus.
                            return;
                        }
                    }

                } //catch(InputException inputex)

                // Get the state of the device.
                try { joystickState = applicationDevice.CurrentJoystickState; }
                // Catch any exceptions. None will be handled here, 
                // any device re-aquisition will be handled above.  
                catch (InputException)
                {
                    return;
                }

                if (UpdateEvent != null)
                {
                    UpdateEvent(this);
                }
        }

    }
}
