using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using Hslu.Bluetooth;
using InTheHand.Net.Sockets;
using InTheHand.Net;
using System.IO;
using RobotCtrl;

namespace InputController
{
    public partial class XBoxController : Form
    {
        private bool mouseDown = false;
        private double arc = 0.0;
        private double speed = 0.0;

        private StreamWriter writer;
        private BluetoothClient client;

        private string lastSerializedTrack;

        private int currentGear = 1;
        private int gearOffset = 50;

        private JoystickInputController joystick;
        delegate void Invoker(JoystickInputController joystick);

        public XBoxController()
        {
            InitializeComponent();

            FillComboInputController();

            this.Focus();
        }

        private void FillComboInputController()
        {
            comboInputController.Items.Clear();
            comboInputController.Items.Add("Keyboard");
            comboInputController.SelectedIndex = 0;
            foreach (var pair in JoystickInputController.EnumDevices())
            {
                comboInputController.Items.Add(pair.Value);
            }
        }

        private void XBoxController_KeyDown(object sender, KeyEventArgs e)
        {
            //WriteLog("KeyDown: "+e.KeyCode);
            switch (e.KeyCode)
            {
                case Keys.A:
                    AButton.Visible = true;
                    break;
                case Keys.B:
                    BButton.Visible = true;
                    break;
                case Keys.S:
                    StartButton.Visible = true;
                    StartButton_Click(null, EventArgs.Empty);
                    break;
                case Keys.R:
                    LeftShoulder.Visible = true;
                    GearUp();
                    break;
                case Keys.L:
                    RightShoulder.Visible = true;
                    GearDown();
                    break;
                case Keys.X:
                    XButton.Visible = true;
                    break;
                case Keys.Y:
                    YButton.Visible = true;
                    break;
                
            }
        }

        private void XBoxController_KeyUp(object sender, KeyEventArgs e)
        {
            //WriteLog("KeyUp: " + e.KeyCode);
            switch (e.KeyCode)
            {
                case Keys.A:
                    AButton.Visible = false;
                    break;
                case Keys.B:
                    BButton.Visible = false;
                    break;
                case Keys.R:
                    LeftShoulder.Visible = false;
                    break;
                case Keys.L:
                    RightShoulder.Visible = false;
                    break;
                case Keys.S:
                    //StartButton.Visible = false;
                    break;
                case Keys.X:
                    XButton.Visible = false;
                    break;
                case Keys.Y:
                    YButton.Visible = false;
                    break;
                
            }
        }

        private void WriteLog(string msg)
        {
            WriteLog(msg, Color.Black);
        }

        private void WriteLog(string msg, Color color)
        {
            
            ListViewItem lvi = new ListViewItem(DateTime.Now.ToLongTimeString());
            lvi.ForeColor = color;
            lvi.SubItems.Add(msg);
            listViewLog.Items.Add(lvi);
            listViewLog.EnsureVisible(listViewLog.Items.Count - 1);
        }

        private void RightThumbButton_MouseDown(object sender, MouseEventArgs e)
        {
            mouseDown = true;

            CalcAxisPosition(e.X - (RightThumbButton.Width / 2), (RightThumbButton.Height / 2) - e.Y);
        }

        private void RightThumbButton_MouseUp(object sender, MouseEventArgs e)
        {
            mouseDown = false;
        }

        private void RightThumbButton_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouseDown)
            {
                CalcAxisPosition(e.X - (RightThumbButton.Width / 2), (RightThumbButton.Height / 2) - e.Y);
            }
        }


        private void StartButton_Click(object sender, EventArgs e)
        {
            if (client == null || !client.Connected)
            {
                ConnectBluetooth();
            }
            else
            {
                DisconnectBluetooth();
            }
            
        }

        private void ConnectBluetooth()
        {
            RobotID robotID = RobotIDs.Robot09;
            //Guid serviceID = new Guid("{FB4B43E4-0328-4056-82A5-7E03BE347082}");

            try
            {
                client = new BluetoothClient();

                // connect to desired service
                client.Connect(robotID.Mac, robotID.Guid);
                WriteLog("Client " + client.Client.LocalEndPoint + " tries to connect to "+robotID.Mac);

                writer = new StreamWriter(client.GetStream());

                StartButton.Visible = true;
            }
            catch (Exception ex)
            {
                WriteLog(ex.Message, Color.Red);
                StartButton.Visible = false;
            }
        }

        private void DisconnectBluetooth()
        {
            try
            {
                //Stop motor
                SendCommand(new TrackPause(10));
                SendCommand("exit");

                writer.Close();
                writer = null;
                client.Close();
                WriteLog("BT connection closed");

                StartButton.Visible = false;
            }
            catch (Exception ex)
            {
                WriteLog(ex.Message, Color.Red);
                StartButton.Visible = false;
            }
        }

        

        /// <summary>
        /// Send Track commands
        /// </summary>
        /// <param name="track"></param>
        private void SendCommand(Track track)
        {
            string serializedTrack = TrackSerialization.Serialize(track);

            if (serializedTrack != lastSerializedTrack)
            {
                if (writer != null)
                {
                    try
                    {
                        WriteLog(serializedTrack);
                        writer.WriteLine(serializedTrack);
                        writer.Flush();
                    }
                    catch (Exception ex)
                    {
                        WriteLog(ex.Message, Color.Red);
                    }
                }
                else
                {
                    WriteLog(serializedTrack, Color.LightGray);
                }

                lastSerializedTrack = serializedTrack;
            }
        }

        /// <summary>
        /// Send special commands
        /// </summary>
        /// <param name="cmd"></param>
        private void SendCommand(string cmd)
        {
            if (writer != null)
            {
                try
                {
                    WriteLog(cmd);
                    writer.WriteLine(cmd);
                    writer.Flush();
                }
                catch (Exception ex)
                {
                    WriteLog(ex.Message, Color.Red);
                }
            }
        }

        private void comboInputController_SelectedIndexChanged(object sender, EventArgs e)
        {
            if ((string)comboInputController.SelectedItem == "Keyboard")
            {
                RightThumbButton.Visible = true;
                this.joystick = null;
            }
            else
            {
                RightThumbButton.Visible = true;
                this.joystick = new JoystickInputController();
                this.joystick.ConnectInputDevice((string)comboInputController.SelectedItem);
                this.joystick.UpdateEvent += new JoystickInputController.UpdateEventHandler(joystick_UpdateEvent);
                
            }
        }

        void joystick_UpdateEvent(JoystickInputController controller)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new Invoker(joystick_UpdateEvent), controller);
                return;
            }
            
            CalcAxisPosition(joystick.XAxis1, joystick.ZAxis1 * (-1));

            PushButtons(joystick.Buttons);
        }

        private byte[] buttonsLastState;

        private void checkButtonState(byte[] oldVal, byte[] newVal, int index, Keys key){
            if (oldVal[index] != newVal[index])
            {
                if (newVal[index] == 128)
                {
                    XBoxController_KeyDown(null, new KeyEventArgs(key));
                }
                else
                {
                    XBoxController_KeyUp(null, new KeyEventArgs(key));
                }
            }
        }

        private void PushButtons(byte[] buttons)
        {
            if (buttonsLastState != null)
            {
                checkButtonState(buttonsLastState, buttons, 0, Keys.A);
                checkButtonState(buttonsLastState, buttons, 1, Keys.B);
                checkButtonState(buttonsLastState, buttons, 2, Keys.X);
                checkButtonState(buttonsLastState, buttons, 3, Keys.Y);
                checkButtonState(buttonsLastState, buttons, 4, Keys.R);
                checkButtonState(buttonsLastState, buttons, 5, Keys.L);
                checkButtonState(buttonsLastState, buttons, 7, Keys.S);
            }
            buttonsLastState = buttons;

        }

        /// <summary>
        /// Game Controllers usually have a black range where they return inaccurate values
        /// </summary>
        /// <param name="value"></param>
        /// <param name="range"></param>
        /// <returns></returns>
        private void RangeCorrection(ref int value, int range)
        {
            if (value < range && value > -range)
            {
                value = 0;
            }
            else if (value >= range)
            {
                value -= range;
            }
            else
            {
                value += range;
            }
        }

        private void CalcAxisPosition(int newAngle, int newSpeed)
        {
            //Handle joystick inaccurance
            RangeCorrection(ref newAngle, 10);
            RangeCorrection(ref newSpeed, 1);

            //Send Track commands
            if (newSpeed != 0)
            {
                speed = (double)newSpeed / (double)numSpeedCalibration.Value;

                if (newAngle != 0)
                {
                    arc = (double)numArcCalibration.Value / (double)newAngle;

                    //Radius correction: the faster the car, the higher the radius
                    arc += (speed * 0.05);   

                    if (arc > 0)
                    {
                        SendCommand(new TrackArcRight(arc, 360, speed, 1.0));
                    }
                    else if (arc < 0)
                    {
                        SendCommand(new TrackArcLeft(arc * (-1), 360, speed, 1.0));
                    }
                }
                else
                {
                    arc = 0.0;
                    SendCommand(new TrackLine(1000, speed, 1.0));
                }
            }
            else
            {
                speed = 0.0;
                SendCommand(new TrackPause(1000));
            }

            if (speed > 1.0)
            {
                joystick.SetVibration();
            }
            //Update GUI information
            lblArc.Visible = true;
            lblArc.Text = "Arc=" + newAngle;
            
            
            //lblRightThumbButton.Visible = true;
            //lblRightThumbButton.Text = "X=" + newAngle + " Y=" + newSpeed;

            if (newSpeed > 0)   //forward speed
            {
                LeftTriggerProgress.Visible = false;
                RightTriggerProgress.Visible = true;
                RightTriggerProgress.Height = Math.Abs(newSpeed);
                lblFwSpeed.Visible = true;
                lblFwSpeed.Text = "Forward Speed=" + Math.Abs(newSpeed);
            }
            else if(newSpeed < 0) //backward speed
            {
                RightTriggerProgress.Visible = false;
                LeftTriggerProgress.Visible = true;
                LeftTriggerProgress.Height = Math.Abs(newSpeed);
                lblBwSpeed.Visible = true;
                lblBwSpeed.Text = "Backward Speed=" + Math.Abs(newSpeed);
            }
            else
            {
                LeftTriggerProgress.Height = 0;
                RightTriggerProgress.Height = 0;
                lblFwSpeed.Text = "";
                lblBwSpeed.Text = "";
            }
            
            
            //Just to test performance:
            joystick.PollingInterval = (int)numPollingInterval.Value;
        }



        /// <summary>
        /// GearUp manipulates the speed correction values to make the car faster
        /// </summary>
        private void GearUp()
        {
            if(currentGear<4)
            {
                currentGear++;
                numSpeedCalibration.Value -= gearOffset;
            }
        }

        /// <summary>
        /// GearDown manipulates the speed correction values to make the car slower
        /// </summary>
        private void GearDown()
        {
            if (currentGear > 1)
            {
                currentGear--;
                numSpeedCalibration.Value += gearOffset;
            }
        }

    }
}