﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Collections.Specialized;

namespace RobotCtrl
{
    
    public class Console
    {
        private Timer timer;
        public delegate void SwitchesChangedEventHandler(BitVector32 oldValue, BitVector32 newValue);
        public event SwitchesChangedEventHandler SwitchesChanged;
        
        private int switchesLastData;
        private DigitalIn switches;
        public DigitalIn Switches
        {
            get { return switches; }
        }

        private DigitalOut lamps;
        public DigitalOut Lamps
        {
            get { return lamps; }
        }

        /// <summary>
        /// Default construction for virtual environment
        /// </summary>
        public Console()
            :this(Config.IsWinCE ? RunMode.REAL : RunMode.VIRTUAL)
        {}

        /// <summary>
        /// Constructor to chose between virtual and real runmodes
        /// </summary>
        /// <param name="mode"></param>
        public Console(RunMode mode)
        {
            if (mode == RunMode.VIRTUAL)
            {
                switches = new DigitalIn();
                lamps = new DigitalOut();
            }
            else
            {
                switches = new DigitalIn_HW(Config.IOConsoleSwitches);
                lamps = new DigitalOut_HW(Config.IOConsoleLeds);
            }

            //Poll switches
            TimerCallback cb = new TimerCallback(PollSwitches);
            timer = new Timer(cb, null, 0, 100);
        }

        private void PollSwitches(object state)
        {
            
            int newData = switches.Data;
            if (newData != switchesLastData)
            {
                int oldData = this.switchesLastData;
                this.switchesLastData = newData;
                if (SwitchesChanged != null)
                {
                    SwitchesChanged(new BitVector32(oldData), new BitVector32(newData));
                }
            }
        }

        public event EventHandler<OverrideLampArgs> OverrideLamp;

        public void OverrideLampInvocation(int bitIndex, bool state)
        {
            //override value of selected lamp
            this.lamps[bitIndex] = state;

            //throw event to ConsoleView to update the lamp image
            if (OverrideLamp != null)
            {
                OverrideLamp(this, new OverrideLampArgs(bitIndex, state));
            }
        }



    }


    public class OverrideLampArgs : System.EventArgs
    {
        private int bitIndex;
        public int BitIndex
        {
            get { return bitIndex; }
        }
        private bool state;
        public bool State
        {
            get { return state; }
        }

        public OverrideLampArgs(int bitIndex, bool state)
        {
            this.bitIndex = bitIndex;
            this.state = state;
        }
    } 
}
