﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Collections;

namespace RobotCtrl
{
    public static class TrackSerialization
    {
        public static Track Deserialize(string data)
        {
            string[] parts = data.Split(':');

            Type type = Type.GetType("RobotCtrl." + parts[0]);
            ConstructorInfo ctor = type.GetConstructors().Single();

            var parameters = new ArrayList();
            ParameterInfo[] ctorParamInfos = ctor.GetParameters();
            for (int i = 0; i < ctorParamInfos.Length; i++)
            {
                object value = Convert.ChangeType(parts[i + 1], ctorParamInfos[i].ParameterType, null);
                parameters.Add(value);
            }
            return (Track)ctor.Invoke(parameters.ToArray());
        }


        //   Constructor Signatures:
        // public TrackPause(double pauseTimeSeconds)
        // public TrackLine(double length, double speed, double acceleration)
        // public TrackArcLeft(double radius, double angle, double speed, double acceleration)
        // public TrackArcRight(double radius, double angle, double speed, double acceleration)
        // public TrackTurn(double angle, double speed, double acceleration)
        private static Dictionary<Type, Func<Track, string[]>> lookup = new Dictionary<Type, Func<Track, string[]>>()
        {
            {typeof(TrackPause), track => new string[]{ track.pauseTime.ToString() } },
            {typeof(TrackLine), track => new string[]{ track.length.ToString(), track.NominalSpeedNegative.ToString(), track.acceleration.ToString() } },
            {typeof(TrackArcLeft), track => new string[]{ ((TrackArcLeft)track).radius.ToString(), ((TrackArcLeft)track).angle.ToString(),
                                                            track.NominalSpeedNegative.ToString(), track.acceleration.ToString() } },
            {typeof(TrackArcRight), track => new string[]{ ((TrackArcRight)track).radius.ToString(), ((TrackArcRight)track).angle.ToString(),
                                                            track.NominalSpeedNegative.ToString(), track.acceleration.ToString() } },
            {typeof(TrackTurn), track => new string[]{ ((TrackTurn)track).angle.ToString(),
                                                            track.NominalSpeedNegative.ToString(), track.acceleration.ToString() } }
        };

        public static string Serialize(Track track)
        {
            string[] parameters = lookup[track.GetType()](track);
            return track.GetType().Name + ":" + String.Join(":", parameters);
        }
    }
}
