using System;
using System.Collections.Generic;
using System.Text;
using ServerPattern;
using System.IO;
using System.Net.Sockets;

namespace FileServer
{
    class FileHandler : AbstractHandler
    {
        private Socket socket;
        private StreamReader reader;
        private StreamWriter writer;
        private string fileName;

        public FileHandler(Socket client)
            : base(client)
        {
            this.socket = client;
            var stream = new NetworkStream(client, true);
            reader = new StreamReader(stream);
            writer = new StreamWriter(stream);
        }

        override protected Boolean ReadRequest()
        {
            string firstLine = reader.ReadLine();
            if (!firstLine.StartsWith("GET"))
                return false;

            fileName = firstLine.Split(' ')[1].Substring(1); // Substring removes leading slash
            if (fileName == "favicon.ico")
            {
                return false;
            }
            else if (fileName == "exit")
            {
                socket.Close();
                Console.WriteLine("Socket closed");
                return false;
            }
            else if (String.IsNullOrEmpty(fileName) || fileName.StartsWith("index"))
            {
                fileName = "commandLog.txt";
            }
            return true;
        }

        override protected void CreateResponse()
        {
            string content = "";//File.ReadAllText(Path.Combine(Directory.GetCurrentDirectory(), fileName));
            using (var file = new StreamReader(fileName))
            {
                content = file.ReadToEnd();
            }
            writer.WriteLine("HTTP/1.0 200 OK");
            writer.WriteLine("Content-type: text/plain");
            writer.WriteLine("Content-length: ", content.Length);
            writer.WriteLine("");
            writer.Write(content);
            writer.Flush();
        }
    }
}
